import os
import shutil


def delete_file(path):
    os.remove(path)


def create_file(path):
    if not os.path.exists(get_parent(path)):
        os.makedirs(get_parent(path))
    fd = os.open(path, os.O_CREAT)
    os.close(fd)


def file_exist(file_path):
    return os.path.isfile(file_path)


def directory_exist(directory_path):
    return os.path.isdir(directory_path)


def move(source, destination):
    shutil.move(source, destination)


def create_dir(directory_path):
    os.makedirs(directory_path)


def delete_directory(directory_path):
    shutil.rmtree(directory_path)


def directory_is_empty(directory_path):
    return not get_directory_content(directory_path)


def get_directory_content(directory_path):
    if not path_is_directory(directory_path):
        raise PathIsNotDirectoryException(directory_path)

    return os.listdir(directory_path)


def get_biggest_files(dir_path, root_path):
    # traverse up a folder structure. Finding the biggest file
    while True:
        biggest_file = get_biggest_file(dir_path)

        if biggest_file.exist():
            yield biggest_file

        if dir_path.strip('/') == root_path.strip('/'):
            break

        dir_path = get_parent(dir_path)


def get_biggest_file(dir_path):
    class BiggestFile:
        def __init__(self):
            self.size = -1
            self.path = None

        def exist(self):
            return self.size != -1

    biggest_file = BiggestFile()

    for file in get_directory_content(dir_path):
        file = dir_path + '/' + file
        if path_is_directory(file):
            continue
        file_size = get_file_size(file)
        if biggest_file.size < file_size:
            biggest_file.size = get_file_size(file)
            biggest_file.path = file

    return biggest_file


def get_files(path):
    if not path_is_directory(path):
        yield path
        return

    for root, dirs, files in os.walk(path):
        for name in files:
            yield root + '/' + name


def path_is_directory(path):
    return os.path.isdir(path)


def get_file_type(path):
    return os.path.splitext(path)[1]


def get_last_path_part(path):
    if not path:
        return path

    if path[-1] == '/':
        path = path[:-1]
    return os.path.basename(path)


def get_last_name_from_path(path):
    path_removed_extension = get_path_without_extension(path)
    return get_last_path_part(path_removed_extension)


def get_path_without_extension(path):
    if path_is_directory(path):
        return path
    return os.path.splitext(path)[0]


# Get the file's parent directory
def get_parent(path):
    return os.path.dirname(path)


def get_file_size(path):
    return os.path.getsize(path)


class PathIsNotDirectoryException(Exception):

    def __init__(self, path):
        super(PathIsNotDirectoryException, self).__init__(path + ' is not a directory')


from memover import logger
from memover.arguments_parser import MeMoverArgs


class PathQueue:

    def __init__(self, args: MeMoverArgs):
        self.__queue = []
        self.__args = args

    def __eq__(self, other):
        if isinstance(other, list):
            return self.__queue == other
        return self is other

    def is_empty(self):
        return not self.__queue

    def next(self):
        return self.__queue[0]

    def remove(self, path):
        self.__queue.remove(path)

    def append(self, path):
        cleaned_path = self.__clean_path(path)
        if cleaned_path in self.__queue:
            return
        self.__queue.append(cleaned_path)

    def log_debug(self, queue_name):
        logger.debug(f'{queue_name}: {self.__queue}')

    def in_queue(self, path):
        path = self.__clean_path(path)
        for queue_path in self.__queue:
            if queue_path == path:
                return True
        return False

    def __clean_path(self, path):
        everything_after_source = self.__lstrip_source(path)
        everything_after_source = everything_after_source.lstrip('/')

        try:
            slash_index = everything_after_source.index('/')
            first_root_path = everything_after_source[:slash_index]
        except ValueError:
            first_root_path = everything_after_source

        first_root_path = first_root_path.rstrip('/')
        source = self.__args.source.rstrip('/')
        return f'{source}/{first_root_path}'

    def __lstrip_source(self, path):
        index = path.index(self.__args.source) + len(self.__args.source)
        return path[index:]



import logging

Log = logging.getLogger("me-mover")


def setup(level: int):
    logging.basicConfig(
        level=level,
        format="%(asctime)s %(levelname)-8s %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S")

    logging.getLogger('watchdog').setLevel(logging.INFO)


def info(message):
    Log.info(message)


def debug(message):
    Log.debug(message)



from . import file_handler


def search_files_with_file_type(search, path, file_type):
    files = search_files(search, path)
    return [found_file for found_file in files if file_handler.get_file_type(found_file) == file_type]


def search_files(search, path):
    files = []
    keywords = search.split()  # Split on whitespace
    for file_name in file_handler.get_directory_content(path):
        file_path = path + '/' + file_name

        match = __match_file(keywords, file_name)
        if match:
            files.append(file_path)

    return files


def __match_file(keywords, file_name):
    for keyword in keywords:
        if not __is_keyword_part_of_filename(keyword, file_name):
            return False
    return True


def __is_keyword_part_of_filename(keyword, file_name):
    return keyword.lower() in file_name.lower()  # keyword is part of name




import argparse
import logging
from enum import Enum


class Commands(Enum):
    BY_NAME = 1
    BY_PATH = 2
    WATCH = 3


class MeMoverArgs:
    def __init__(
            self,
            type: Commands,
            source: str,
            show_destination: str,
            movie_destination: str,
            media_name: str = None,
            **flags
    ):
        self.__type = type
        self.__source = source
        self.__show_destination = show_destination
        self.__movie_destination = movie_destination
        self.__media_name = media_name
        self.__log_level = flags.get('log_level', None)
        self.__moves_before_quit = flags.get('moves_before_quit', None)
        super().__init__()

    @property
    def type(self) -> Commands:
        return self.__type

    @property
    def source(self) -> str:
        return self.__source

    @property
    def show_destination(self) -> str:
        return self.__show_destination

    @property
    def movie_destination(self) -> str:
        return self.__movie_destination

    @property
    def log_level(self) -> int:
        return self.__log_level

    @property
    def moves_before_quit(self) -> int:
        return self.__moves_before_quit

    @property
    def media_name(self) -> str:
        return self.__media_name

    def __str__(self) -> str:
        return f'type: {self.__type}, source: {self.__source}, show_destination: {self.__show_destination}, movie_destination: {self.__movie_destination}, moves_before_quit: {self.__moves_before_quit}, media_name: {self.__media_name}'


class MeMoverArgsCreator:

    def __init__(self):
        self.__args = None

    def generate_args(self):
        parser = argparse.ArgumentParser()
        subparsers = parser.add_subparsers(help="commands")

        self.__create_name(subparsers)
        self.__create_path(subparsers)
        self.__create_watch(subparsers)

        args = parser.parse_args()
        if hasattr(args, 'func'):
            args.func(args)
        else:
            parser.print_help()

    @property
    def args(self):
        return self.__args

    def __set_to_name(self, args):
        self.__args = MeMoverArgs(Commands.BY_NAME, args.source, args.show_destination, args.movie_destination, args.media_name, log_level=args.verbose)

    def __set_to_path(self, args):
        self.__args = MeMoverArgs(Commands.BY_PATH, args.source, args.show_destination, args.movie_destination, log_level=args.verbose)

    def __set_to_watch(self, args):
        self.__args = MeMoverArgs(Commands.WATCH, args.source, args.show_destination, args.movie_destination, log_level=args.verbose, moves_before_quit=args.moves)

    def __create_name(self, subparsers):
        parser_name = subparsers.add_parser('by-name')
        parser_name.add_argument(
            'media_name',
            metavar='name',
            help='name of TV show or movie to move'
        )
        parser_name.add_argument(
            'source',
            metavar='source',
            help='source directory to look for media in'
        )
        self.__add_common_arguments(parser_name)
        parser_name.set_defaults(func=self.__set_to_name)

    def __create_path(self, subparsers):
        parser_path = subparsers.add_parser('by-path')
        parser_path.add_argument(
            'source',
            metavar='source',
            help='source path of movie or tv-show to move'
        )
        self.__add_common_arguments(parser_path)

        parser_path.set_defaults(func=self.__set_to_path)

    def __create_watch(self, subparsers):
        parser_watch = subparsers.add_parser('watch')
        parser_watch.add_argument(
            'source',
            metavar='source',
            help='source directory to to watch for incoming TV shows and movies'
        )
        self.__add_common_arguments(parser_watch)
        parser_watch.add_argument('--moves', type=int, required=False, dest='moves', help='Number of moves before exit')
        parser_watch.set_defaults(func=self.__set_to_watch)

    @staticmethod
    def __add_common_arguments(parser):
        parser.add_argument(
            'show_destination',
            metavar='shows-destination',
            help='show destination directory'
        )
        parser.add_argument(
            'movie_destination',
            metavar='movies-destination',
            help='movie destination directory'
        )

        parser.add_argument(
            "-v", "--verbose",
            help="Increase output verbosity",
            action="store_const",
            const=logging.DEBUG,
            default=logging.INFO
        )

me_mover_args_creator = MeMoverArgsCreator()


def get_args() -> MeMoverArgs:
    if not me_mover_args_creator.args:
        me_mover_args_creator.generate_args()
    return me_mover_args_creator.args


if __name__ == '__main__':
    current_args = get_args()
    print(current_args)



__author__ = 'alexander.berlind'

from setuptools import setup, find_packages

with open("README.md", "r") as fh:
      long_description = fh.read()

setup(name='me-mover',
      version='3.1.2',
      packages=find_packages(exclude=['tests', 'tests.*']),
      author='Alexander Berlind',
      url='https://github.com/Alecktos/me-mover.git',
      description="Organize movies and tv-shows",
      long_description=long_description,
      long_description_content_type="text/markdown",
      license='MIT',
      entry_points={
            'console_scripts': ['me-mover = memover.main:main']
      },
      install_requires=[
            'watchdog'
      ],
      python_requires='>=3.8'
)



import os

from memover import mover, logger
from memover.arguments_parser import MeMoverArgs
from memover.watcher.path_queue import PathQueue


class SyncedWatcher:

    def __init__(self, args: MeMoverArgs):
        self.__modified_paths_queue = PathQueue(args)
        self.__created_paths_queue = PathQueue(args)
        self.__args = args

    def move_next_path(self):
        if self.__created_paths_queue.is_empty():
            return False

        path = self.__created_paths_queue.next()

        if self.__modified_paths_queue.in_queue(path):
            self.__modified_paths_queue.remove(path)
            return False

        self.__log_queue_status('Queues Before Move: Before removing path')

        # Remove file from queue before moving to avoid changes to file in modify queue
        self.__created_paths_queue.remove(path)

        self.__log_queue_status('Queues Before Move: after removing path')
        mover.move_media_by_path(
            path,
            self.__args.show_destination,
            self.__args.movie_destination
        )
        self.__log_queue_status('Queues After Move')
        return True

    def __log_queue_status(self, header):
        logger.debug(f'-- {header} --')
        self.__modified_paths_queue.log_debug('modified_paths')
        self.__created_paths_queue.log_debug('created_paths')
        logger.debug('-- End --')

    # Created_paths_to_move

    @property
    def created_paths(self):
        return self.__created_paths_queue

    def on_created(self, path):
        self.__created_paths_queue.append(path)
        self.on_modified(path)

    # Modified_paths

    @property
    def modified_paths(self):
        return self.__modified_paths_queue

    def on_modified(self, path):
        if path.strip('/') == self.__args.source.strip('/'):
            return

        if not os.path.exists(path):
            return

        if not self.__created_paths_queue.in_queue(path):
            return

        self.__modified_paths_queue.append(path)
